Summary
Types
Anything compatible with Curves.define_bezier/3.
See function docs for details.
The values to be randomly picked from.
A list of floats, representing percentages, that should add up to 1.0
The weight, as given by the end user, which will influence the randomization.
Types
Anything compatible with Curves.define_bezier/3.
@type opts() :: keyword()
See function docs for details.
@type outcomes() :: [...] | Range.t()
The values to be randomly picked from.
@type probabilities() :: [float()]
A list of floats, representing percentages, that should add up to 1.0
@type weight_spec() :: %{ :target => any(), required(:amount | :weight) => number(), optional(:left_dist) => integer(), optional(:right_dist) => integer(), optional(:radius) => integer(), optional(:curve) => curve_spec() }
The weight, as given by the end user, which will influence the randomization.